import * as z from './base';
import { ZodUndefined } from './undefined';
import { ZodNull } from './null';
import { ZodUnion } from './union';
export interface ZodShapeDef<T extends z.ZodRawShape = z.ZodRawShape> extends z.ZodTypeDef {
    t: z.ZodTypes.object;
    shape: T;
}
export interface ZodObjectDef<T extends z.ZodRawShape = z.ZodRawShape> extends z.ZodTypeDef {
    t: z.ZodTypes.object;
    shape: T;
    strict: boolean;
}
declare type OptionalKeys<T extends z.ZodRawShape> = {
    [k in keyof T]: undefined extends T[k]['_type'] ? k : never;
}[keyof T];
declare type RequiredKeys<T extends z.ZodRawShape> = Exclude<keyof T, OptionalKeys<T>>;
declare type ObjectIntersection<T extends z.ZodRawShape> = {
    [k in OptionalKeys<T>]?: T[k]['_type'];
} & {
    [k in RequiredKeys<T>]: T[k]['_type'];
};
declare type Flatten<T extends z.ZodRawShape> = {
    [k in keyof T]: T[k];
};
declare type ObjectType<T extends z.ZodRawShape> = Flatten<ObjectIntersection<T>>;
declare type MergeObjectParams<First extends ZodObjectParams, Second extends ZodObjectParams> = {
    strict: First['strict'] extends false ? false : Second['strict'] extends false ? false : true;
};
interface ZodObjectParams {
    strict: boolean;
}
export declare class ZodObject<T extends z.ZodRawShape, Params extends ZodObjectParams = {
    strict: true;
}> extends z.ZodType<Params['strict'] extends true ? ObjectType<T> : Flatten<ObjectType<T> & {
    [k: string]: any;
}>, // { [k in keyof T]: T[k]['_type'] },
ZodObjectDef<T>> {
    readonly _params: Params;
    toJSON: () => {
        t: z.ZodTypes.object;
        shape: {
            [x: string]: any;
        }[];
    };
    nonstrict: () => ZodObject<T, Flatten<{ [k in Exclude<keyof Params, "strict">]: Params[k]; } & {
        strict: false;
    }>>;
    /**
     * Prior to zod@1.0.12 there was a bug in the
     * inferred type of merged objects. Please
     * upgrade if you are experiencing issues.
     */
    merge: <MergeShape extends z.ZodRawShape, MergeParams extends ZodObjectParams>(other: ZodObject<MergeShape, MergeParams>) => ZodObject<T & MergeShape, MergeObjectParams<Params, MergeParams>>;
    optional: () => ZodUnion<[this, ZodUndefined]>;
    nullable: () => ZodUnion<[this, ZodNull]>;
    static create: <T_1 extends z.ZodRawShape>(shape: T_1) => ZodObject<T_1, {
        strict: true;
    }>;
}
export {};
